package core;

import java.util.ArrayList;

public class Monomio{
  public byte[] valor;
  public Monomio(int n)
  {
    valor=new byte[n];
  }
  public Monomio(int n, byte val)
  {
    valor=new byte[n];
    for(int i=0;i<n;i++) this.valor[i]=val;
  }
  public Monomio(byte[] valores)
  {
    valor=valores.clone();
  }
  public boolean equals(Object a)
  {
    int i;
    if (!(a instanceof Monomio)) return false;
    for(i=0;i<valor.length; i++)
      if (valor[i] != ((Monomio)a).valor[i])
        return false;
    return true;
  }
  
  public int qtdeLiterais() {
	  int count=0;
	  for(int i=0; i<valor.length; i++) if (this.valor[i]!=0) count++;
	  return count;
  }
  
  public int getTamanho() {return this.valor.length;}
  
  /*
   * Retorna o Monomio que só pode ser satisfeito se ambos this e x forem satisfeitos 
   */
  public Monomio and(Monomio x)
  {
	Monomio resul=new Monomio(valor.length);
	for(int i=0; i<valor.length; i++)
		if(this.valor[i]==-x.valor[i] && this.valor[i]!=0)
			return new Monomio(valor.length, (byte)0);
		else if (this.valor[i]!=0)
			resul.valor[i]=this.valor[i];
		else
			resul.valor[i]=x.valor[i];
	  
	return resul;
  }
  
  /*
   * Retorna true se this e x nunca podem ser satisfeitos ao mesmo tempo
   */
  public boolean independente(Monomio x) {
	  for(int i=0; i<valor.length; i++)
		  if (this.valor[i]==-x.valor[i] && this.valor[i]!=0)
			  return true;
	  return false;
  }
  
  /*
   * Retorna true se this e x tem alguma variável em comum
   */
  public boolean varComum(Monomio x) {
	  for(int i=0; i<valor.length; i++)
		  if (x.valor[i]!=0 && this.valor[i]!=0)
			  return true;
	  return false;
  }
  
  /*
   * Retorna um conjunto de Monomio que devem independentes desse que juntos satisfazem
   * as mesmas interpretações que x  
   */
  public ArrayList<Monomio> tornarIndependente(Monomio x)
  {
	  ArrayList<Monomio> result;
	  if (this.independente(x)){
		  result=new ArrayList<Monomio>(1);
		  result.add(x);
	  }
	  else {
		  //Esse "Monômio" servirá apenas para guardar os literais de this que não estão em x
		  Monomio L=new Monomio(this.getTamanho(), (byte)0);
		  boolean LVazio=true;
		  for (int i=0; i<valor.length; i++)
			  if(this.valor[i]!=0 && x.valor[i]==0) {
				  L.valor[i]=this.valor[i];
				  LVazio=false;
			  }
		  if (LVazio) return new ArrayList<Monomio>(1); //this "contém" x
		  else {
			  result=new ArrayList<Monomio>();
			  Monomio U, W=new Monomio(x.valor);
			  for (int i = 0; i<this.valor.length; i++) {
				  if (L.valor[i]!=0) {
					  U=new Monomio(W.valor);
					  U.valor[i]=(byte) -L.valor[i];
					  result.add(U);
					  W.valor[i]=L.valor[i];
				  }
			  }
		  }
	  }
	  return result;
  }
  
  public String toString()
  {
    int i;
    String res = "( ";
    for(i=0;i<valor.length; i++)
        res += valor[i]+" ";
    res += ")";
    return res;
  }
}
