package plugins;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;

import core.Monomio;

public class Dubois implements Plugin {
	public static final String info="Implementa o método de contagem apresentado por Olivier Dubois em 1991 no artigo " +
	"intitulado 'Counting the number of solutions for instances of satisfiability'.\nEssa versão utiliza a procedure " +
	"'independence' apresentada no artigo para tornar as conjunções independentes.";
	public static final String nome="Método Olivier Dubois usando a procedure 'independence'";
	public long iteracoes=0;
	public long getIteracoes() {
		return iteracoes;
	}
	public BigInteger contaInterp(int vars, ArrayList<Monomio> formula) {
		if (vars==0) return BigInteger.ZERO;
		BigInteger result=BigInteger.ZERO;
		ArrayList<Monomio> formulaIndependete=independece(formula);
		for (Iterator<Monomio> it=formulaIndependete.iterator(); it.hasNext(); ){
			result=result.add(BigInteger.ONE.shiftLeft(vars-(it.next()).qtdeLiterais()));
			iteracoes++;
		}
		return result;
	}

	protected ArrayList<Monomio> independece(ArrayList<Monomio> formula) {
		ArrayList<Monomio> result=new ArrayList<Monomio>();
		ArrayList<Monomio> formulaTmp=new ArrayList<Monomio>();
		ArrayList<Monomio> tmp;
		try {
			result.add(formula.get(0));
			formulaTmp.add(formula.get(0));
			for (int i=1; i<formula.size(); i++) {
				tmp=ik(formulaTmp, formula.get(i));
				result.addAll(tmp);
				formulaTmp.add(formula.get(i));
			}
			return result;
		} catch (java.lang.IndexOutOfBoundsException e) {
			return new ArrayList<Monomio>(0);
		}
	}

	private ArrayList<Monomio> ik(ArrayList<Monomio> formulaTmp, Monomio x) {
		ArrayList<Monomio> Gamma=new ArrayList<Monomio>();
		ArrayList<Monomio> result=new ArrayList<Monomio>();
		ArrayList<Monomio> formulaChamada=new ArrayList<Monomio>(formulaTmp.subList(0, formulaTmp.size()-1));
		Gamma=formulaTmp.get(formulaTmp.size()-1).tornarIndependente(x);
		if (formulaTmp.size()>1) {
			if (Gamma.size()>0) for(Iterator<Monomio> it=Gamma.iterator(); it.hasNext(); ) result.addAll(ik(formulaChamada, it.next()));
		} else
			result.addAll(Gamma);
		iteracoes++;
		return result;
	}

	public Monomio pegaInterp(int vars, ArrayList<Monomio> formula) {
		return null;
	}

}
