package plugins;

import java.util.*;
import java.math.BigInteger;
import core.Monomio;

public class TabelaVerdade implements Plugin {
	public static final String info="Método da tabela verdade.";
	public static final String nome="Tabela verdade";
	public long iteracoes=0;

	public long getIteracoes() {
		return iteracoes;
	}

	public BigInteger contaInterp(int vars, ArrayList<Monomio> formula) {
		return geraCombinacoesConta(vars, formula, java.math.BigInteger.ZERO, new byte[vars]);
	}
	
	private BigInteger geraCombinacoesConta(int n, ArrayList<Monomio> formula, BigInteger sat, byte[] valores)
    {
        if (n==0)
        {
            if (testa_formula(formula, valores))
                sat=sat.add(java.math.BigInteger.ONE);
        }       
        else
        {
            valores[valores.length - n]=-1;
            sat = geraCombinacoesConta(n-1, formula, sat, valores);
            valores[valores.length - n]=1;
            sat = geraCombinacoesConta(n-1, formula, sat, valores);
        }
        iteracoes++;
        return sat;
    }
	
	private byte[] geraCombinacoesInterp(int n, ArrayList<Monomio> formula, byte[] valores)
    {
		byte[] respostas;
		if (n==0)
        {
            if (testa_formula(formula, valores))
                return valores;
        }       
        else
        {
            valores[valores.length - n]=-1;
            if((respostas = geraCombinacoesInterp(n-1, formula, valores))!=null)
            		return respostas;
            valores[valores.length - n]=1;
            if((respostas = geraCombinacoesInterp(n-1, formula, valores))!=null)
        		return respostas;
        }
        return null;
    }
    
    private boolean testa_formula(ArrayList<Monomio> formula, byte[] valores)
    {
    	Monomio x;
    	int j;
    	monomios:
    	for(Iterator<Monomio> i=formula.iterator(); i.hasNext();) {
    		x=i.next();
    		if (x.valor.length==0) continue; //Clausas vazias são sempre falsas.
    		for(j=0; j<x.valor.length; j++)
    			if(x.valor[j]==-valores[j])
    				continue monomios;
    		return true;
    	} 
    	return false;
    }

	public Monomio pegaInterp(int vars, ArrayList<Monomio> formula) {
		return new Monomio(geraCombinacoesInterp(vars, formula, new byte[vars]));
	}
}
